# Generated by Django 2.2.22 on 2021-05-16 17:02

from django.db import migrations, models
from django_jsonfield_backport.models import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('scrud_django', '0004_auto_20210306_1600'),
    ]

    operations = [
        migrations.AlterField('Resource', 'content', models.fields.TextField()),
        migrations.AlterField('Resource', 'content', JSONField()),
    ]